@@ -2,7 +2,7 @@
 /*
  * ibus - The Input Bus
  *
- * Copyright (c) 2007-2008 Huang Peng <shawn.p.huang@gmail.com>
+ * Copyright (c) 2007-2014 Huang Peng <shawn.p.huang@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 #include <QInputMethodEvent>
 #include <QTextCharFormat>
 #include <QApplication>
+#include <QLineEdit>
 #include <qibus.h>
 #include "ibus-input-context.h"
 
@@ -87,8 +88,10 @@ IBusInputContext::IBusInputContext (const BusPointer &bus)
       m_preedit_visible (false),
       m_preedit_cursor_pos (0),
       m_has_focus (false),
-      m_caps (CapPreeditText | CapFocus),
-      m_n_compose (0)
+      m_password_mode (false),
+      m_caps (CapPreeditText | CapFocus | CapSurroundingText),
+      m_n_compose (0),
+      m_needs_surrounding_text (false)
 {
     Q_ASSERT (!m_bus.isNull ());
 
@@ -162,7 +165,33 @@ IBusInputContext::update (void)
     QPoint topleft = widget->mapToGlobal(QPoint(0,0));
     rect.translate (topleft);
 
-    m_context->setCursorLocation (rect.x (), rect.y (), rect.width (), rect.height ());
+    m_context->setCursorLocation (rect.x (),
+                                  rect.y (),
+                                  rect.width (),
+                                  rect.height ());
+
+    if (m_needs_surrounding_text) {
+        QString surroundingText =
+                widget->inputMethodQuery (Qt::ImSurroundingText).toString ();
+        uint cursor_pos =
+                widget->inputMethodQuery (Qt::ImCursorPosition).toUInt ();
+        uint anchor_pos =
+                widget->inputMethodQuery (Qt::ImAnchorPosition).toUInt ();
+
+        Text *wrappedSurroundingText = new Text (surroundingText);
+        TextPointer wrappedSurroundingTextPointer (wrappedSurroundingText);
+
+        m_context->setSurroundingText (wrappedSurroundingTextPointer,
+                                       cursor_pos,
+                                       anchor_pos);
+
+        // We don't destroy wrappedSurroundingText because when
+        // wrappedSurroundingTextPointer is destroyed when this function
+        // returns, it calls wrappedSurroundingText->unref()
+        // which, in turn, destroys wrappedSurroundingText.
+        // It would actually crash if we did since wrappedSurroundingText
+        // is already freed.
+    }
 
 #if 0
     QVariant value;
@@ -196,6 +225,16 @@ IBusInputContext::setFocusWidget (QWidget *widget)
     if (m_context.isNull ())
         return;
 
+    m_password_mode = false;
+
+    if (m_has_focus && widget->inherits ("QLineEdit")) {
+        QLineEdit::EchoMode mode =
+                qobject_cast<QLineEdit *>(widget)->echoMode();
+        if (mode == QLineEdit::Password || mode == QLineEdit::NoEcho) {
+            m_password_mode = true;
+        }
+    }
+
     if (m_has_focus) {
         m_context->focusIn ();
     }
@@ -236,9 +275,13 @@ translate_x_key_event (XEvent *xevent, uint *keyval, uint *keycode, uint *state)
         *state |= IBus::ReleaseMask;
 
     char key_str[64];
-    if (XLookupString (&xevent->xkey, key_str, sizeof (key_str), (KeySym *)keyval, 0) <= 0) {
-        *keyval = (quint32) XLookupKeysym (&xevent->xkey, 0);
-    }
+    /* Follow gtkxim module.
+     * https://git.gnome.org/browse/gtk+/tree/modules/input/gtkimcontextxim.c#n736
+     * I think XLookupString can get right keyval even if the return value
+     * is 0 but key_str might not be a string.
+     * And XLookupString won't return the negative value in my test. */
+    XLookupString (&xevent->xkey, key_str, sizeof (key_str),
+                   (KeySym *)keyval, 0);
 
     return true;
 
@@ -251,6 +294,10 @@ IBusInputContext::x11FilterEvent (QWidget *keywidget, XEvent *xevent)
     uint keycode = 0;
     uint state = 0;
 
+    if (m_password_mode) {
+        return false;
+    }
+
     if (!m_has_focus) {
         m_has_focus = true;
         if (!m_context.isNull ())
@@ -536,6 +583,10 @@ IBusInputContext::createInputContext (void)
             this, SLOT (slotShowPreeditText (void)));
     connect (m_context, SIGNAL (hidePreeditText (void)),
             this, SLOT (slotHidePreeditText (void)));
+    connect (m_context, SIGNAL (deleteSurroundingText (int, uint)),
+            this, SLOT (slotDeleteSurroundingText (int, uint)));
+    connect (m_context, SIGNAL (requireSurroundingText (void)),
+            this, SLOT (slotRequireSurroundingText (void)));
 
     if (m_has_focus) {
         m_context->focusIn ();
@@ -722,11 +773,22 @@ IBusInputContext::displayPreeditText (const TextPointer &text, uint cursor_pos,
 void
 IBusInputContext::slotUpdatePreeditText (const TextPointer &text, uint cursor_pos, bool visible)
 {
+    // set visible to false, if text is empty
+    visible = visible && !text->text ().isEmpty ();
+
+    // set cursor at end, if pos is greater than the text length
+    if (cursor_pos > (uint)text->text ().length ())
+        cursor_pos = text->text ().length ();
+
+    bool update = (m_preedit_visible != visible) || visible;
+
     m_preedit = text;
     m_preedit_visible = visible;
     m_preedit_cursor_pos = cursor_pos;
 
-    displayPreeditText (m_preedit, m_preedit_cursor_pos, visible);
+    if (update) {
+        displayPreeditText (m_preedit, m_preedit_cursor_pos, visible);
+    }
 }
 
 void
@@ -760,3 +822,32 @@ IBusInputContext::slotDisconnected (void)
     displayPreeditText (m_preedit, m_preedit_cursor_pos, false);
     deleteInputContext ();
 }
+
+void
+IBusInputContext::slotDeleteSurroundingText (int offset, uint nchars)
+{
+
+    QWidget *widget = focusWidget();
+    if (widget == NULL)
+        return;
+
+    int cursor = widget->inputMethodQuery(Qt::ImCursorPosition).toInt();
+
+    cursor += offset;
+    if (cursor < 0) {
+        offset -= cursor;
+        nchars += cursor;
+    } 
+
+    QInputMethodEvent event;
+    event.setCommitString ("", offset, nchars);
+    sendEvent (event);
+    update ();
+}
+
+void
+IBusInputContext::slotRequireSurroundingText (void)
+{
+    m_needs_surrounding_text = true;
+    update();
+}
