@@ -37,6 +37,7 @@ public:
     void setCapabilities (uint caps);
     void setCursorLocation (int x, int y, int w, int h);
     void setEngine (const QString &name);
+    void setSurroundingText (const TextPointer &text, uint cursor_pos, uint anchor_pos);
 
 public:
     static InputContextPointer create (const BusPointer &bus, const QString &name);
@@ -67,6 +68,8 @@ Q_SIGNALS:
     void disabled (void);
 
     void forwardKeyEvent (uint keyval, uint keycode, uint state);
+    void deleteSurroundingText (int offset, uint nchars);
+    void requireSurroundingText (void);
 
 private Q_SLOTS:
     void slotCommitText (const QDBusVariant &text);
@@ -94,6 +97,8 @@ private Q_SLOTS:
     void slotDisabled();
 
     void slotForwardKeyEvent(uint keyval, uint keycode, uint state);
+    void slotDeleteSurroundingText(int offset, uint nchars);
+    void slotRequireSurroundingText();
 
 /*
 signals:
